<?php
/**
 * @category    Payment
 * @author      Global Payments Europe s.r.o. (emailgpwebpay@gpe.cz)
 */

defined('_JEXEC') or die('Restricted access');

require_once dirname(__FILE__) . '/Muzo/gpmuzo.php';

class pm_gpwebpay extends PaymentRoot {

  function __construct() {
    $jshopConfig = JSFactory::getConfig();

    JSFactory::loadExtLanguageFile('pm_gpwebpay');
  }

  function showPaymentForm($params, $pmconfigs){
    include(dirname(__FILE__)."/paymentform.php");
  }

  static function sendToLog($message) {
    saveToLog("payment.log", $message);
  }

  //function call in admin
  function showAdminFormParams($params){
    $array_params = array(
      'gp_webpay_url',
      'merchant_number',
      'public_key_file',
      'private_key_file',
      'private_key_password',
      'deposit_flag',
      'payment_number_offset',
      'transaction_end_status'
    );
    
    foreach ($array_params as $key){
      if (!isset($params[$key])) $params[$key] = '';
    }
    $orders = JModelLegacy::getInstance('orders', 'JshoppingModel'); //admin model
    include(dirname(__FILE__)."/adminparamsform.php");
  }

  function checkTransaction($pmconfigs, $order, $act){
      return array(1, ''); //vždy ok. Nejsou metody na zpětné ověření na platební bráně
  }

  function showEndForm($pmconfigs, $order){
    $session = JFactory::getSession();
    $session->set('gp_order', $order->order_id);

    $GpWebPayUrlDetail = $this->getGpWebPayUrl($pmconfigs, $order);

    if(strlen($GpWebPayUrlDetail['url']) != 0){
        
        echo _JSHOP_REDIRECT_TO_PAYMENT_PAGE; ?>
            <form id="paymentform" action="<?php echo $GpWebPayUrlDetail['url'] ?>" name="paymentform" method="post">
            </form>
            <script>
                document.getElementById('paymentform').submit();
            </script>
            <?php
        
    }
    else{
        $cancel_return = JURI::root()."index.php?option=com_jshopping&controller=checkout&task=step7&act=cancel&js_paymentclass=pm_gpwebpay";
        Header("Location: ".$cancel_return."");
    }
    die;
  }

  function getUrlParams($pmconfigs){
    $session = JFactory::getSession();
    $params = array();
    $params['order_id'] = $session->get('gp_order');
    $params['hash'] = "";
    $params['checkHash'] = 0;
    $params['checkReturnParams'] = 1; //je zpětná kontrola vždy, pokud je 0 tak se objednavka neuloží
    return $params;
  }
  
  public function getGpWebPayUrl($config, $order){
      $response = array();
      $response['url'] = "";
      $response['error'] = false;
      $response['text'] = "";
      $certPath = dirname(__FILE__) . '/Certs/';
      
      if (!empty($config['gp_webpay_url'])) {
          $gateway = $config['gp_webpay_url'];
      } 
      else 
      {
          $response['error'] = true;
          $response['text'] = "Empty gateway URL";
      }
      
      if (!empty($config['merchant_number'])) {
          $merchantNumber = $config['merchant_number'];
      }
      else 
      {
          $response['error'] = true;
          $response['text'] = "Empty merchant number";
      }
      
      if (!empty($config['public_key_file'])) {
          $publicKeyFilename = $config['public_key_file'];
          if(!file_exists($certPath . $publicKeyFilename))
          {
              $response['error'] = true;
              $response['text'] = "Public key not found";
          }
      } 
      else
      {
          $response['error'] = true;
          $response['text'] = "Public key is empty";
      }
      
      if (!empty($config['private_key_file'])) {
          $privateKeyFilename = $config['private_key_file'];
          if(!file_exists($certPath . $privateKeyFilename))
          {
              $response['error'] = true;
              $response['text'] = "Private key not found";
          }
      } 
      else 
      {
          $response['error'] = true;
          $response['text'] = "Private key is empty";
      }
      
      if (!empty($config['private_key_password'])) {
          $privateKeyPassword = $config['private_key_password'];
      } 
      else 
      {
          $response['error'] = true;
          $response['text'] = "Password for private key is empty";
      }
      
      if ($config['deposit_flag'] == '1' || $config['deposit_flag'] == '0') {
          $transferType = $config['deposit_flag'];
      } 
      else 
      {
          $response['error'] = true;
          $response['text'] = "Transfer type not set";
      }
      
      if (!empty($config['payment_number_offset'])) {
          $orderIdFrom = $config['payment_number_offset'];
      } 
      else 
      {
          $orderIdFrom = 0;
      }
      
      $amount = $order->order_total;
      
      $orderCurrency = $order->currency_code;
      $currencyCode = array(  'CZK' => '203', //Česká koruna 
                              'EUR' => '978', //Euro
                              'GBP' => '826', //Pound sterling
                              'USD' => '840', //US dollar     
                              'HRK' => '191', //Chorvatská kuna
                              'CAD' => '124', //Canadian Dollar
                              'SEK' => '752', //Swedish Krona 
                              'RUB' => '643', //Russian ruble  
                              'PLN' => '985', //Polish zloty  
                              'RSD' => '941', //Serbian Dinar  
                              'NOK' => '578', //Norwegian Krone
                              'BGN' => '975', //Bulgarian Lev
                              'HUF' => '348', //Forint
                              'CHF' => '756', //Swiss Franc
                              'DKK' => '208', //Danish Krone
                              'RON' => '946', //Romanian new leu
                              'TRY' => '949'  //Turkish Lira
                          );
      $currencyCodeIso4217 = null; 
      if(strlen($currencyCode[$orderCurrency]) == 0)
      {
          $response['error'] = true;
          $response['text'] = "Currency ".$orderCurrency." is not allowed";
      }
      else
      {
          $currencyCodeIso4217 = $currencyCode[$orderCurrency];
          $amountInTransaction = $amount * 100;
      }
      $gpTransactionId = $order->order_id;
      $gpTransactionGpWebPayId = $order->order_id + $orderIdFrom;
      
      $returnUrl = JURI::root()."index.php?option=com_jshopping&controller=gpwebpay&task=checkResponse&js_paymentclass=pm_gpwebpay";
  
      $description = "Payment id ".$gpTransactionId;
      $userEmail = $order->email;
      
      $response['url'] = GpMuzo_CreateOrderExt(// funkce presmeruje browser s pozadavkem na server Muzo     
          $gateway, // adresa kam posilat pozadavek do Muzo
          $returnUrl, // adresa kam ma Muzo presmerovat odpoved
          $certPath . $privateKeyFilename, // soubor s privatnim klicem
          $privateKeyPassword, // heslo privatniho klice
          $merchantNumber, // cislo obchodnika
          $gpTransactionGpWebPayId, // cislo objednavky
          $amountInTransaction, // hodnota objednavky v halerich
          $currencyCodeIso4217, // kod meny, CZK..203, EUR..978, GBP..826, USD..840, povolene meny zalezi na smlouve s bankou
          $transferType, // uhrada okamzite "1", nebo uhrada az z admin rozhrani
          $gpTransactionId, // identifikace objednavky pro obchodnika
          $description, // popis nakupu, pouze ASCII
          "X", // data obchodnika, pouze ASCII
          '', //language
          $userEmail //email
      );
      
      if(strlen($response['url']) == 0)
      {
          $response['error'] = true;
          $response['text'] = "Error in creating URL";
      }
      
      return $response;
    }
}
?>
